/** Panel in which all the user inputs the necessary values to 
	run the genetic algorithm for the CECS 375 Final Project.
    @author Dan Kelly
	@version 1.0
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class EntryPanel extends JPanel {

	private GeneticFrame gf;
	private JTextField memField;
	private JTextArea memArea;
	private JButton memButton = new JButton("Set Number of Members");
	
	private JTextField fitField;
	private JTextArea fitArea;
	private JButton fitButton = new JButton("Set Fitness Threshold");
	
	private JTextField crossField;
	private JTextArea crossArea;
	private JButton crossButton = new JButton("Set Crossover Rate");
	
	private JTextField mutField;
	private JTextArea mutArea;
	private JButton mutButton = new JButton("Set Mutation Rate");
	
	private JButton runButton = new JButton("Evolution");
	
	/** Creates a panel with labels, buttons, and text boxes for user input.*/
	public EntryPanel (GeneticFrame geneticFrame) {
		setBackground(Color.black);
		gf = geneticFrame;
		setLayout(new BorderLayout());
		
		Box box = Box.createVerticalBox();
		
		// Members
		JLabel memLabel = new JLabel("Number of Members");
		memLabel.setForeground(Color.yellow);
		box.add(memLabel);
		
		memField = new JTextField(12);
		memField.setBackground(Color.black);
		memField.setForeground(Color.yellow);
		memField.setCaretColor(Color.yellow);
		memField.setSelectedTextColor(Color.yellow);
		memField.setSelectionColor(Color.darkGray);
		box.add(memField);
		
		box.add(memButton);
		memButton.addActionListener(new LocalActionListener());
		
		box.add(Box.createRigidArea(new Dimension(1,2)));
		
		// Fitness Threshold
		JLabel fitLabel = new JLabel("Fitness Threshold");
		fitLabel.setForeground(Color.yellow);
		box.add(fitLabel);
		
		fitField = new JTextField(12);
		fitField.setBackground(Color.black);
		fitField.setForeground(Color.yellow);
		fitField.setCaretColor(Color.yellow);
		fitField.setSelectedTextColor(Color.yellow);
		fitField.setSelectionColor(Color.darkGray);
		box.add(fitField);
		
		box.add(fitButton);
		fitButton.addActionListener(new LocalActionListener());
		
		box.add(Box.createRigidArea(new Dimension(1,2)));
		
		// Cross-over rate
		JLabel crossLabel = new JLabel("Crossover Rate");
		crossLabel.setForeground(Color.yellow);
		box.add(crossLabel);
		
		crossField = new JTextField(12);
		crossField.setBackground(Color.black);
		crossField.setForeground(Color.yellow);
		crossField.setCaretColor(Color.yellow);
		crossField.setSelectedTextColor(Color.yellow);
		crossField.setSelectionColor(Color.darkGray);
		box.add(crossField);
		
		box.add(crossButton);
		crossButton.addActionListener(new LocalActionListener());
		
		box.add(Box.createRigidArea(new Dimension(1,2)));
		
		// Mutation Rate
		JLabel mutLabel = new JLabel("Mutation Rate");
		mutLabel.setForeground(Color.yellow);
		box.add(mutLabel);
		
		mutField = new JTextField(12);
		mutField.setBackground(Color.black);
		mutField.setForeground(Color.yellow);
		mutField.setCaretColor(Color.yellow);
		mutField.setSelectedTextColor(Color.yellow);
		mutField.setSelectionColor(Color.darkGray);
		box.add(mutField);
		
		box.add(mutButton);
		mutButton.addActionListener(new LocalActionListener());
		
		box.add(Box.createRigidArea(new Dimension(1,2)));
		
		// Run Generations
		box.add(runButton);
		runButton.addActionListener(new LocalActionListener());
		
		add(box);
	} // End Constructor
	
	/** This method is invoked each time
    	the panel has to be repainted. It
    	should not be invoked directly. */
	public void paintComponent (Graphics g) {
		super.paintComponent(g);
	} // End paintComponent
	
	class LocalActionListener implements ActionListener {
		public void actionPerformed(ActionEvent evt) {
    		if((evt.getSource() == memButton) && !(memField.getText().equals("")) ) {
				gf.createPopulation((Integer.valueOf(memField.getText())).intValue());
			} // End if
			
			if((evt.getSource() == fitButton) && !(fitField.getText().equals("")) ) {
				gf.setFitnessThreshold((Integer.valueOf(fitField.getText())).intValue());
			} // End if
			
			if((evt.getSource() == crossButton) && !(crossField.getText().equals("")) ) {
				gf.setCrossoverRate((Integer.valueOf(crossField.getText())).intValue());
			} // End if
			
			if((evt.getSource() == mutButton) && !(mutField.getText().equals("")) ) {
				gf.setMutationRate((Integer.valueOf(mutField.getText())).intValue());
			} // End if
				
			else if(evt.getSource() == runButton) {
				gf.runGenerations();
			} // End else if
					
		} // End actionPerformed
	} // End LocalActionListener
	
} // End EntryPanel