/** Frame for CECS 375 Final Project
    @author Dan Kelly
	@version 1.0
*/

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class GeneticFrame extends JFrame {

  	private Container container;
	private GeneticPanel geneticPanel = new GeneticPanel(this);
	private EntryPanel entryPanel = new EntryPanel(this);
	private FitnessPanel fitnessPanel = new FitnessPanel(this);
	private ResultPanel resultPanel = new ResultPanel(this);
	
	private JMenuBar menuBar = new JMenuBar();
	private JMenu fileMenu = new JMenu("File");
	private JMenuItem exit = new JMenuItem("Exit");
	
	private Vector memberVector = new Vector(0);
	private Vector tempVector = new Vector(0);
	
	private int numGenerations = 0;
	private double fitnessThreshold = 0;
	private double crossoverRate = 0;
	private double mutationRate = 0;
	private double strongFit = 0;
		

	/** Set the frame in the middle of the screen.  The frame is 
	    half the size of the screen.  It contains one 
		instance each of EntryPanel, GeneticPanel, and FitnessPanel. */
	public GeneticFrame () {

		setTitle("Genetic Algorithm");
		setResizable(false);
    
    	// Get the toolkit and dimension so the screen size can be received 
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension d = tk.getScreenSize();
		int screenWidth = d.width;
		int screenHeight = d.height;
		
		setLocation(screenWidth/4,screenHeight/4);  
		setSize(512,384);
		
		// Close when the X is clicked
		addWindowListener(new WindowAdapter()
			{public void windowClosing (WindowEvent e)
				{System.exit(0);}});
				
    	// Panels
		container = getContentPane();		
		container.add(geneticPanel,BorderLayout.EAST);
		container.add(entryPanel,BorderLayout.WEST);
		container.add(fitnessPanel,BorderLayout.SOUTH);
		
		// Menu Bar
		menuBar.add(fileMenu);
		fileMenu.add(exit);
		exit.addActionListener(new LocalActionListener());
		setJMenuBar(menuBar);
		
		
  	} // END Constructor
  	
  	/** Creates a random population of <i>n</i> size.
      	@param n The number of members to include in the population. 
		*/
  	public void createPopulation(int n) {
  		memberVector.clear();
  		numGenerations = 0;
  		strongFit = 0;
		for(int i = 0; i < n; i++) {
			Member m = new Member();
			m.setAttributes();
			memberVector.addElement(m);
		} // End for
		container.remove(geneticPanel);
		geneticPanel = new GeneticPanel(this);
		container.add(geneticPanel,BorderLayout.EAST);
		container.remove(fitnessPanel);
		fitnessPanel = new FitnessPanel(this);
		container.add(fitnessPanel,BorderLayout.SOUTH);
		container.validate();
	} // End createGeneration
	
	/** Returns the number of members.*/
	public int getNumMembers() {
		return memberVector.size();	
	} // End getNumMembers
	
	/** Evolves the population by computing new generations using the Select, Crossover, and Mutation methods.*/
	public void runGenerations() {
		
		strongFit = 0;
		numGenerations = 0;
		
		double fit = getPopFitness();

		while(getStrongestMemberFitness() < fitnessThreshold) {
			//System.out.println("New Generation\n");
			//System.out.println("fit = "+fit+"\n");
			
			// Select
			Select();
			
			// Crossover
			Crossover();
			
			// Mutation
			Mutation();
			
			// Update Member Vector
			Vector memberVector = new Vector(0);
			for(int i = 0; i < tempVector.size(); i++) {
				memberVector.addElement((Member)(tempVector.elementAt(i)));
			} // End for
			tempVector.clear();
					
			fit = getPopFitness();
			numGenerations++;
			
			//System.out.println("Population Fitness = "+fit+"\n");
			
		}  // End while
		
		//System.out.println("Fitness Threshold Acheived.");
		
		container.remove(fitnessPanel);
		fitnessPanel = new FitnessPanel(this);
		container.add(fitnessPanel,BorderLayout.SOUTH);
		container.validate();
		
		showResultPanel();
		
	} // End runGenerations	
	
	/** Returns the number of generations it took to find a member which had a fitness greater thatn or equal to the fitness threshold..*/
	public int getNumGenerations() {
		return numGenerations;
	} // End getNumGenerations 
	
	/** Returns the average population fitness.*/
	public double getPopFitness() {
		double fit = 0;
		for(int i = 0; i < memberVector.size(); i++) {
			fit += ((Member)(memberVector.elementAt(i))).fitness();
		} // End for
		fit /= memberVector.size();
		
		return fit;
	} // End getPopFitness
	
	/** Sets the fitness threshold.
      	@param n The fitness threshold. 
		*/
	public void setFitnessThreshold(int n) {
		if(n <= 100)
			fitnessThreshold = ((double)n)/100;
			
		container.remove(geneticPanel);
		geneticPanel = new GeneticPanel(this);
		container.add(geneticPanel,BorderLayout.EAST);
		container.validate();
	} // End setFitnessThreshold
	
	/** Sets the crossover rate.
      	@param n The crossover rate. 
		*/
	public void setCrossoverRate(int n) {
		if(n <= 100)
			crossoverRate = ((double)n)/100;
			
		container.remove(geneticPanel);
		geneticPanel = new GeneticPanel(this);
		container.add(geneticPanel,BorderLayout.EAST);
		container.validate();
	} // End setCrossoverRate
	
	/** Sets the mutation rate.
      	@param n The mutation rate. 
		*/
	public void setMutationRate(int n) {
		if(n <= 100)
			mutationRate = ((double)n)/100;
		
		container.remove(geneticPanel);
		geneticPanel = new GeneticPanel(this);
		container.add(geneticPanel,BorderLayout.EAST);
		container.validate();
	} // End setMutationRate
	
	/** Returns the fitness threshold.*/
	public double getFitnessThreshold() {
		return fitnessThreshold;
	} // End getFitnessThreshold
	
	/** Returns the crossover rate.*/
	public double getCrossoverRate() {
		return crossoverRate;
	} // End getCrossoverRate
	
	/** Returns the mutation rate.*/
	public double getMutationRate() {
		return mutationRate;
	} // End getMutationRate
	
	/** Returns the fitness of the strongest member.*/
	public double getStrongFit() {
		return strongFit;
	} // End getStrongFit
	
	/** Returns the strongest member.*/
	public Member getStrongestMember () {
		SortByProbabilities();
		return (Member)(memberVector.elementAt(0));
	} // End  getStrongestMember
	
	private double getStrongestMemberFitness() {
		// Assign Probabilities
		for (int i = 0; i < memberVector.size(); i++){
			((Member)(memberVector.elementAt(i))).setProbability((((Member)(memberVector.elementAt(i))).fitness())/getPopFitness());
		} // End for
			
		SortByProbabilities();
		
		strongFit = ((Member)(memberVector.elementAt(0))).fitness();
		
		//System.out.println("Strongest = "+strongFit+"\n");
		
		return strongFit;
	} // End getStrongestMemberFitness
	
	private void SortByProbabilities() {
		for(int i = 1; i < memberVector.size(); i++) {
			for(int j = 0; j < i; j++) {
				if(((Member)memberVector.elementAt(i)).getProbability() > ((Member)memberVector.elementAt(j)).getProbability()) {
					memberVector.insertElementAt((Member)memberVector.remove(i),j);
					break;
				} // End if
			} // End for
		} // End for
	} // End SortByProbabilities
	
	private void Select() {
		int numSelected = (int)((1-crossoverRate)*memberVector.size());
		//System.out.println("Members to be selected "+numSelected+"\n");
		
		for(int i = 0; i < numSelected; i++) {
			tempVector.addElement((Member)(memberVector.elementAt(i)));
			//System.out.println("Selected\n");
		} // End for
		
	}  // End Select
	
	private void Crossover() {
		int numSelected = (int)(((crossoverRate*memberVector.size())/2)+1);
		//System.out.println("Pairs to be crossed "+numSelected+"\n");
		
		Vector temp = new Vector(0);
		
		for(int i = 0; i < 2*numSelected; i+=2) {
			
			tempVector.addElement((Member)(memberVector.elementAt(i)));
			tempVector.addElement((Member)(memberVector.elementAt(i+1)));
			
			int j = 0;
			
			for(j = 0; j < (Member.NUM_OF_ATTRIBUTES / 2); j++) {
				temp.addElement(new Boolean( (((Member)tempVector.elementAt(i)).getAttributes())[j] ));
			} // End for
			
			int b = 0;
			
			for(b = j; b < Member.NUM_OF_ATTRIBUTES-1; b++) {
				temp.addElement(new Boolean( (((Member)tempVector.elementAt(i+1)).getAttributes())[b] ));
			} // End for
			
			b = j;
			
			for(int k = 0; k < (Member.NUM_OF_ATTRIBUTES / 2); k++) {
				((Member)tempVector.elementAt(i+1)).setAttributeAt(j,((Boolean)(temp.elementAt(0))).booleanValue());
				temp.remove(0);
				j++;
			} // End for
			
			for(int k = 0; k < (Member.NUM_OF_ATTRIBUTES / 2)-1; k++) {
				((Member)tempVector.elementAt(i)).setAttributeAt(k,((Boolean)(temp.elementAt(0))).booleanValue());
				temp.remove(0);
			} // End for
			
			//System.out.println("Cross\n");
			
		} // End for
	} // End Crossover
	
	private void Mutation() {
		int mutSelected = (int)(mutationRate*memberVector.size());
		//System.out.println("Members to be mutated "+mutSelected+"\n");
		
		for(int i = 0; i < mutSelected; i++) {
			int r = (int) (Math.random() * tempVector.size());
			((Member)(tempVector.elementAt(r))).mutate();	
			
			//System.out.println("Mutated\n");
		} // End for
		
	} // End Mutation
	
	private void showResultPanel() {
		
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension d = tk.getScreenSize();
  		int screenWidth = d.width;
  		int screenHeight = d.height;
		
		JFrame resultFrame = new JFrame();
		resultFrame.setSize(220,200);
		resultFrame.setLocation((screenWidth/2)-110,(screenHeight/2)-100);
		resultFrame.setTitle("Results");
		resultFrame.setResizable(false);
		
		Container container = resultFrame.getContentPane();  
    	container.setLayout(new BorderLayout());
		container.add(resultPanel,BorderLayout.CENTER);
		
		resultFrame.show();
		
	} // End showResultPanel
	
	class LocalActionListener implements ActionListener {
		public void actionPerformed (ActionEvent e) {
			if(e.getSource() == exit) {
				System.exit(0);
			} // End if
			
		} // End actionPerformed
		
	} // End LocalActionListener
	
} // END GeneticFrame