/** Member class for CECS 375 Final Project
    @author Dan Kelly
	@version 1.0
*/

import java.lang.Math;

public class Member {

	/** The number of attributes each member has. */
	public static final int NUM_OF_ATTRIBUTES = 16;
	
	private double fit = 0.0;
	private double probability = 0.0;
	private boolean[] attribute = new boolean[NUM_OF_ATTRIBUTES];
	
	/** Creates the member. */
	public void Member() {} // End Constructor
	
	/** Randomly selects a boolean value for the member. */
	public void setAttributes() {
		for (int i = 0; i < NUM_OF_ATTRIBUTES; i++) {
			if(Math.random() >= 0.5)
				attribute[i] = true;
			else
				attribute[i] = false;
		} // End for
	} // End setAttributes
	
	/** Calculates the fitness of the member.
		@return The fitness of the member.
		*/
	public double fitness() {
		
		fit = 0;
		
		for (int i = 0; i < NUM_OF_ATTRIBUTES; i++) {		
			if(attribute[i] == true)
				fit += 1;
		} // End for
			
		fit = (fit/NUM_OF_ATTRIBUTES);
		
		return fit;
		
	} // End fitness
	
	/** Sets the probability associated with the member.  It's probability if calculated in relation to other members in the population.
		@param p The probability associated with the member.
		*/
	public void setProbability(double p) {
		probability = p;
	} // End setProbability
	
	/** Returns the probability of the member.
		@return The probability of the member.
		*/
	public double getProbability() {
		return probability;
	} // End getProbability
	
	/** Returns the boolean attributes of the member.
		@return The array of attributes.
		*/
	public boolean[] getAttributes() {
		return attribute;
	} // End getAttributes
	
	/** Sets the boolean attributes <i>i</i> of the member to <i>b</i>.
		@param i The attribute to be set.
		@param b The boolean value.
		*/
	public void setAttributeAt(int i, boolean b) {
		attribute[i] = b;
	} // End setAttributeAt
	
	/** Mutates the member by randomly toggling one of its attributes.*/
	public void mutate() {
		
		double r = Math.random();
		double divisions = (double)(1/(double)NUM_OF_ATTRIBUTES);
		double current_mark = 0;
		double next_mark = divisions;
		
		for(int i = 0; i < NUM_OF_ATTRIBUTES; i++) {
			if( (r >= current_mark) && (r < next_mark) ) {
				attribute[i] = !attribute[i];
				break;
			} // End if
			current_mark = next_mark;
			next_mark = current_mark + divisions;
		} // End for
		
	} // End mutate
	
	/** Returns a binary representation of the member's attributes.
		@return A string of binary values
		*/
	public String display() {
		
		String rep = "<";
		
		for (int i = 0; i < NUM_OF_ATTRIBUTES; i++) {
			if(attribute[i] == true)
				rep += "1";
			else
				rep += "0";
		} // End for
		
		rep += ">";
		
		return rep;
	
	} // End display

} // End Member